@@ -5,6 +5,7 @@ import requests
 from zeroconf import Zeroconf
 
 from . import __version__
+from .backend import MESSAGE_TYPES
 from .backend.mprm import Mprm
 from .devices.gateway import Gateway
 from .devices.zwave import Zwave
@@ -34,36 +35,36 @@ class HomeControl(Mprm):
     :param gateway_id: Gateway ID (aka serial number), typically found on the label of the device
     :param mydevolo_instance: Mydevolo instance for talking to the devolo Cloud
     :param zeroconf_instance: Zeroconf instance to be potentially reused
-    :param url: URL of the mPRM (typically leave it at default)
     """
 
     def __init__(self, gateway_id: str, mydevolo_instance: Mydevolo, zeroconf_instance: Optional[Zeroconf] = None):
         self._mydevolo = mydevolo_instance
         self._session = requests.Session()
         self._session.headers.update({"User-Agent": f"devolo_home_control_api/{__version__}"})
-
+        self._zeroconf = zeroconf_instance
         self.gateway = Gateway(gateway_id, mydevolo_instance)
-        super().__init__(mydevolo_instance, zeroconf_instance)
 
-        self.gateway.zones = self.get_all_zones()
+        super().__init__()
+        self._grouping()
 
         # Create the initial device dict
         self.devices: Dict = {}
         self._inspect_devices(self.get_all_devices())
 
-        self.device_names = dict(zip([(self.devices[device].settings_property['general_device_settings'].name + "/"
-                                       + self.devices[device].settings_property['general_device_settings'].zone)
-                                      for device in self.devices],
-                                     [self.devices[device].uid for device in self.devices]))
+        self.device_names = {
+            f"{self.devices[device].settings_property['general_device_settings'].name}/\
+              {self.devices[device].settings_property['general_device_settings'].zone}": self.devices[device].uid
+            for device in self.devices
+        }
 
         self.gateway.home_id = get_home_id_from_device_uid(next(iter(self.device_names.values())))
 
-        self.publisher = Publisher([device for device in self.devices])
+        self.publisher = Publisher(self.devices.keys())
 
         self.updater = Updater(devices=self.devices, gateway=self.gateway, publisher=self.publisher)
         self.updater.on_device_change = self.device_change
 
-        threading.Thread(target=self.websocket_connect, name=f"{__class__.__name__}.websocket_connect").start()
+        threading.Thread(target=self.websocket_connect, name=f"{self.__class__.__name__}.websocket_connect").start()
         self.wait_for_websocket_establishment()
 
     @property
@@ -81,9 +82,11 @@ class HomeControl(Mprm):
         """ Get all blinds devices. """
         blinds_devices = []
         for device in self.multi_level_switch_devices:
-            blinds_devices.extend([self.devices[device.uid] for multi_level_switch_property
-                                   in device.multi_level_switch_property
-                                   if multi_level_switch_property.startswith("devolo.Blinds")])
+            blinds_devices.extend([
+                self.devices[device.uid]
+                for multi_level_switch_property in device.multi_level_switch_property
+                if multi_level_switch_property.startswith("devolo.Blinds")
+            ])
         return blinds_devices
 
     @property
@@ -113,12 +116,12 @@ class HomeControl(Mprm):
             devices = [device for device in device_uids if device not in self.devices]
             mode = "add"
             self._inspect_devices([devices[0]])
-            self._logger.debug(f"Device {devices[0]} added.")
+            self._logger.debug("Device %s added.", devices[0])
         else:
             devices = [device for device in self.devices if device not in device_uids]
             mode = "del"
             self.devices.pop(devices[0])
-            self._logger.debug(f"Device {devices[0]} removed.")
+            self._logger.debug("Device %s removed.", devices[0])
         self.updater.devices = self.devices
         return (devices[0], mode)
 
@@ -135,44 +138,41 @@ class HomeControl(Mprm):
         device_uid = get_device_uid_from_element_uid(uid_info['UID'])
         if not hasattr(self.devices[device_uid], "binary_sensor_property"):
             self.devices[device_uid].binary_sensor_property = {}
-        self._logger.debug(f"Adding binary sensor property to {device_uid}.")
-        self.devices[device_uid].binary_sensor_property[uid_info['UID']] = \
-            BinarySensorProperty(session=self._session,
-                                 gateway=self.gateway,
-                                 mydevolo=self._mydevolo,
-                                 element_uid=uid_info['UID'],
-                                 state=bool(uid_info['properties']['state']),
-                                 sensor_type=uid_info['properties']['sensorType'],
-                                 sub_type=uid_info['properties']['subType'])
+        self._logger.debug("Adding binary sensor property to %s.", device_uid)
+        self.devices[device_uid].binary_sensor_property[uid_info['UID']] = BinarySensorProperty(
+            element_uid=uid_info['UID'],
+            state=bool(uid_info['properties']['state']),
+            sensor_type=uid_info['properties']['sensorType'],
+            sub_type=uid_info['properties']['subType'])
 
     def _binary_switch(self, uid_info: dict):
         """ Process BinarySwitch properties. """
         device_uid = get_device_uid_from_element_uid(uid_info['UID'])
         if not hasattr(self.devices[device_uid], "binary_switch_property"):
             self.devices[device_uid].binary_switch_property = {}
-        self._logger.debug(f"Adding binary switch property to {device_uid}.")
-        self.devices[device_uid].binary_switch_property[uid_info['UID']] = \
-            BinarySwitchProperty(session=self._session,
-                                 gateway=self.gateway,
-                                 mydevolo=self._mydevolo,
-                                 element_uid=uid_info['UID'],
-                                 state=bool(uid_info['properties']['state']),
-                                 enabled=uid_info['properties']['guiEnabled'])
+        self._logger.debug("Adding binary switch property to %s.", device_uid)
+        self.devices[device_uid].binary_switch_property[uid_info['UID']] = BinarySwitchProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_binary_switch,
+            state=bool(uid_info['properties']['state']),
+            enabled=uid_info['properties']['guiEnabled'])
 
     def _general_device(self, uid_info: dict):
         """ Process general device setting (gds) properties. """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding general device settings to {device_uid}.")
-        self.devices[device_uid]. \
-            settings_property['general_device_settings'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             events_enabled=uid_info['properties']['settings']['eventsEnabled'],
-                             name=uid_info['properties']['settings']['name'],
-                             zone_id=uid_info['properties']['settings']['zoneID'],
-                             icon=uid_info['properties']['settings']['icon'])
+        self._logger.debug("Adding general device settings to %s.", device_uid)
+        self.devices[device_uid].settings_property['general_device_settings'] = SettingsProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_setting,
+            events_enabled=uid_info['properties']['settings']['eventsEnabled'],
+            name=uid_info['properties']['settings']['name'],
+            zone_id=uid_info['properties']['settings']['zoneID'],
+            icon=uid_info['properties']['settings']['icon'],
+            zones=self.gateway.zones)
+
+    def _grouping(self):
+        """ Get all zones (also called rooms). """
+        self.gateway.zones = self.get_all_zones()
 
     def _humidity_bar(self, uid_info: dict):
         """
@@ -186,17 +186,14 @@ class HomeControl(Mprm):
         if not hasattr(self.devices[device_uid], "humidity_bar_property"):
             self.devices[device_uid].humidity_bar_property = {}
         if self.devices[device_uid].humidity_bar_property.get(fake_element_uid) is None:
-            self.devices[device_uid].humidity_bar_property[fake_element_uid] = \
-                HumidityBarProperty(session=self._session,
-                                    gateway=self.gateway,
-                                    mydevolo=self._mydevolo,
-                                    element_uid=fake_element_uid,
-                                    sensorType="humidityBar")
+            self.devices[device_uid].humidity_bar_property[fake_element_uid] = HumidityBarProperty(
+                element_uid=fake_element_uid,
+                sensorType="humidityBar")
         if uid_info['properties']['sensorType'] == "humidityBarZone":
-            self._logger.debug(f"Adding humidity bar zone property to {device_uid}.")
+            self._logger.debug("Adding humidity bar zone property to %s.", device_uid)
             self.devices[device_uid].humidity_bar_property[fake_element_uid].zone = uid_info['properties']['value']
         elif uid_info['properties']['sensorType'] == "humidityBarPos":
-            self._logger.debug(f"Adding humidity bar position property to {device_uid}.")
+            self._logger.debug("Adding humidity bar position property to %s.", device_uid)
             self.devices[device_uid].humidity_bar_property[fake_element_uid].value = uid_info['properties']['value']
 
     def _inspect_devices(self, devices: list):
@@ -207,44 +204,10 @@ class HomeControl(Mprm):
             self.devices[device_properties['UID']] = Zwave(mydevolo_instance=self._mydevolo, **properties)
             self.devices[device_properties['UID']].settings_property = {}
             threading.Thread(target=self.devices[device_properties['UID']].get_zwave_info,
-                             name=f"{__class__.__name__}.{self.devices[device_properties['UID']].uid}").start()
-
-        elements = {"devolo.BinarySensor": self._binary_sensor,
-                    "devolo.BinarySwitch": self._binary_switch,
-                    "devolo.Blinds": self._multi_level_switch,
-                    "devolo.DewpointSensor": self._multi_level_sensor,
-                    "devolo.Dimmer": self._multi_level_switch,
-                    "devolo.HumidityBarValue": self._humidity_bar,
-                    "devolo.HumidityBarZone": self._humidity_bar,
-                    "devolo.LastActivity": self._last_activity,
-                    "devolo.Meter": self._meter,
-                    "devolo.MildewSensor": self._binary_sensor,
-                    "devolo.MultiLevelSensor": self._multi_level_sensor,
-                    "devolo.MultiLevelSwitch": self._multi_level_switch,
-                    "devolo.RemoteControl": self._remote_control,
-                    "devolo.SirenMultiLevelSwitch": self._multi_level_switch,
-                    "devolo.ShutterMovementFI": self._binary_sensor,
-                    "devolo.ValveTemperatureSensor": self._multi_level_sensor,
-                    "devolo.VoltageMultiLevelSensor": self._multi_level_sensor,
-                    "devolo.WarningBinaryFI": self._binary_sensor,
-                    "acs.hdm": self._automatic_calibration,
-                    "bas.hdm": self._binary_async,
-                    "bss.hdm": self._binary_sync,
-                    "lis.hdm": self._led,
-                    "gds.hdm": self._general_device,
-                    "cps.hdm": self._parameter,
-                    "mas.hdm": self._multilevel_async,
-                    "mss.hdm": self._multilevel_sync,
-                    "ps.hdm": self._protection,
-                    "sts.hdm": self._switch_type,
-                    "stmss.hdm": self._multilevel_sync,
-                    "trs.hdm": self._temperature_report,
-                    "vfs.hdm": self._led
-                    }
+                             name=f"{self.__class__.__name__}.{self.devices[device_properties['UID']].uid}").start()
 
         # List comprehension gets the list of uids from every device
-        nested_uids_lists = [(uid['properties'].get("settingUIDs")
-                              + uid['properties']['elementUIDs'])
+        nested_uids_lists = [(uid['properties'].get("settingUIDs") + uid['properties']['elementUIDs'])
                              for uid in devices_properties]
 
         # List comprehension gets all uids into one list to make one big call against the mPRM
@@ -253,7 +216,8 @@ class HomeControl(Mprm):
         device_properties_list = self.get_data_from_uid_list(uid_list)
 
         for uid_info in device_properties_list:
-            elements.get(get_device_type_from_element_uid(uid_info['UID']), self._unknown)(uid_info)
+            message_type = MESSAGE_TYPES.get(get_device_type_from_element_uid(uid_info['UID']), "_unknown")
+            getattr(self, message_type)(uid_info)
             try:
                 uid = self.devices[get_device_uid_from_element_uid(uid_info['UID'])]
             except KeyError:
@@ -261,19 +225,20 @@ class HomeControl(Mprm):
             uid.pending_operations = uid.pending_operations or bool(uid_info['properties'].get("pendingOperations"))
 
         # Last activity messages sometimes arrive before a device was initialized and therefore need to be handled afterwards.
-        [self._last_activity(uid_info) for uid_info in device_properties_list
-            if uid_info['UID'].startswith("devolo.LastActivity")]
+        [  # pylint: disable=expression-not-assigned
+            self._last_activity(uid_info)
+            for uid_info in device_properties_list
+            if uid_info['UID'].startswith("devolo.LastActivity")
+        ]
 
     def _automatic_calibration(self, uid_info: dict):
         """ Process automatic calibration (acs) properties. """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding automatic calibration setting to {device_uid}")
-        self.devices[device_uid].settings_property['automatic_calibration'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             calibration_status=bool(uid_info['properties']['calibrationStatus']))
+        self._logger.debug("Adding automatic calibration setting to %s.", device_uid)
+        self.devices[device_uid].settings_property['automatic_calibration'] = SettingsProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_setting,
+            calibration_status=bool(uid_info['properties']['calibrationStatus']))
 
     def _binary_sync(self, uid_info: dict):
         """
@@ -281,22 +246,18 @@ class HomeControl(Mprm):
         setting property, so it is named nicely.
         """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding binary sync setting to {device_uid}")
-        self.devices[device_uid].settings_property['movement_direction'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             inverted=uid_info['properties']['value'])
+        self._logger.debug("Adding binary sync setting to %s.", device_uid)
+        self.devices[device_uid].settings_property['movement_direction'] = SettingsProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_setting,
+            inverted=uid_info['properties']['value'])
 
     def _binary_async(self, uid_info: dict):
         """ Process binary async setting (bas) properties. """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding binary async settings to {device_uid}.")
-        settings_property = SettingsProperty(session=self._session,
-                                             gateway=self.gateway,
-                                             mydevolo=self._mydevolo,
-                                             element_uid=uid_info['UID'],
+        self._logger.debug("Adding binary async settings to %s.", device_uid)
+        settings_property = SettingsProperty(element_uid=uid_info['UID'],
+                                             setter=self.set_setting,
                                              value=uid_info['properties']['value'])
 
         # The siren needs to be handled differently, as otherwise their binary async setting will not be named nicely
@@ -325,32 +286,26 @@ class HomeControl(Mprm):
     def _led(self, uid_info: dict):
         """ Process LED information setting (lis) and visual feedback setting (vfs) properties. """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding led settings to {device_uid}.")
+        self._logger.debug("Adding led settings to %s.", device_uid)
         try:
             led_setting = uid_info['properties']['led']
         except KeyError:
             led_setting = uid_info['properties']['feedback']
-        self.devices[device_uid].settings_property['led'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             led_setting=led_setting)
+        self.devices[device_uid].settings_property['led'] = SettingsProperty(element_uid=uid_info['UID'],
+                                                                             setter=self.set_setting,
+                                                                             led_setting=led_setting)
 
     def _meter(self, uid_info: dict):
         """ Process meter properties. """
         device_uid = get_device_uid_from_element_uid(uid_info['UID'])
         if not hasattr(self.devices[device_uid], "consumption_property"):
             self.devices[device_uid].consumption_property = {}
-        self._logger.debug(f"Adding consumption property to {device_uid}.")
-        self.devices[device_uid].consumption_property[uid_info['UID']] = \
-            ConsumptionProperty(session=self._session,
-                                gateway=self.gateway,
-                                mydevolo=self._mydevolo,
-                                element_uid=uid_info['UID'],
-                                current=uid_info['properties']['currentValue'],
-                                total=uid_info['properties']['totalValue'],
-                                total_since=uid_info['properties']['sinceTime'])
+        self._logger.debug("Adding consumption property to %s.", device_uid)
+        self.devices[device_uid].consumption_property[uid_info['UID']] = ConsumptionProperty(
+            element_uid=uid_info['UID'],
+            current=uid_info['properties']['currentValue'],
+            total=uid_info['properties']['totalValue'],
+            total_since=uid_info['properties']['sinceTime'])
 
     def _multilevel_async(self, uid_info: dict):
         """ Process multilevel async setting (mas) properties. """
@@ -364,13 +319,10 @@ class HomeControl(Mprm):
             else:
                 raise
 
-        self._logger.debug(f"Adding {name} setting to {device_uid}.")
-        self.devices[device_uid].settings_property[name] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             value=uid_info['properties']['value'])
+        self._logger.debug("Adding %s setting to %s.", name, device_uid)
+        self.devices[device_uid].settings_property[name] = SettingsProperty(element_uid=uid_info['UID'],
+                                                                            setter=self.set_setting,
+                                                                            value=uid_info['properties']['value'])
 
     def _multilevel_sync(self, uid_info: dict):
         """ Process multilevel sync setting (mss) properties. """
@@ -378,102 +330,84 @@ class HomeControl(Mprm):
 
         # The siren needs to be handled differently, as otherwise their multilevel sync setting will not be named nicely.
         if self.devices[device_uid].device_model_uid == "devolo.model.Siren":
-            self._logger.debug(f"Adding tone settings to {device_uid}.")
-            self.devices[device_uid].settings_property['tone'] = \
-                SettingsProperty(session=self._session,
-                                 gateway=self.gateway,
-                                 mydevolo=self._mydevolo,
-                                 element_uid=uid_info['UID'],
-                                 tone=uid_info['properties']['value'])
+            self._logger.debug("Adding tone settings to %s.", device_uid)
+            self.devices[device_uid].settings_property['tone'] = SettingsProperty(element_uid=uid_info['UID'],
+                                                                                  setter=self.set_setting,
+                                                                                  tone=uid_info['properties']['value'])
 
         # The shutter needs to be handled differently, as otherwise their multilevel sync setting will not be named nicely.
         elif self.devices[device_uid].device_model_uid in ("devolo.model.OldShutter", "devolo.model.Shutter"):
-            self._logger.debug(f"Adding shutter duration settings to {device_uid}.")
-            self.devices[device_uid].settings_property['shutter_duration'] = \
-                SettingsProperty(session=self._session,
-                                 gateway=self.gateway,
-                                 mydevolo=self._mydevolo,
-                                 element_uid=uid_info['UID'],
-                                 shutter_duration=uid_info['properties']['value'])
+            self._logger.debug("Adding shutter duration settings to %s.", device_uid)
+            self.devices[device_uid].settings_property['shutter_duration'] = SettingsProperty(
+                element_uid=uid_info['UID'],
+                setter=self.set_setting,
+                shutter_duration=uid_info['properties']['value'])
 
         # Other devices are up to now always motion sensors.
         else:
-            self._logger.debug(f"Adding motion sensitivity settings to {device_uid}.")
-            self.devices[device_uid].settings_property['motion_sensitivity'] = \
-                SettingsProperty(session=self._session,
-                                 gateway=self.gateway,
-                                 mydevolo=self._mydevolo,
-                                 element_uid=uid_info['UID'],
-                                 motion_sensitivity=uid_info['properties']['value'])
+            self._logger.debug("Adding motion sensitivity settings to %s.", device_uid)
+            self.devices[device_uid].settings_property['motion_sensitivity'] = SettingsProperty(
+                element_uid=uid_info['UID'],
+                setter=self.set_setting,
+                motion_sensitivity=uid_info['properties']['value'])
 
     def _multi_level_sensor(self, uid_info: dict):
         """ Process multi level sensor properties. """
         device_uid = get_device_uid_from_element_uid(uid_info['UID'])
         if not hasattr(self.devices[device_uid], "multi_level_sensor_property"):
             self.devices[device_uid].multi_level_sensor_property = {}
-        self._logger.debug(f"Adding multi level sensor property {uid_info.get('UID')} to {device_uid}.")
-        self.devices[device_uid].multi_level_sensor_property[uid_info['UID']] = \
-            MultiLevelSensorProperty(session=self._session,
-                                     gateway=self.gateway,
-                                     mydevolo=self._mydevolo,
-                                     element_uid=uid_info['UID'],
-                                     value=uid_info['properties']['value'],
-                                     unit=uid_info['properties']['unit'],
-                                     sensor_type=uid_info['properties']['sensorType'])
+        self._logger.debug("Adding multi level sensor property %s to %s.", uid_info['UID'], device_uid)
+        self.devices[device_uid].multi_level_sensor_property[uid_info['UID']] = MultiLevelSensorProperty(
+            element_uid=uid_info['UID'],
+            value=uid_info['properties']['value'],
+            unit=uid_info['properties']['unit'],
+            sensor_type=uid_info['properties']['sensorType'])
 
     def _multi_level_switch(self, uid_info: dict):
         """ Process multi level switch properties. """
         device_uid = get_device_uid_from_element_uid(uid_info['UID'])
         if not hasattr(self.devices[device_uid], "multi_level_switch_property"):
             self.devices[device_uid].multi_level_switch_property = {}
-        self._logger.debug(f"Adding multi level switch property {uid_info.get('UID')} to {device_uid}.")
-        self.devices[device_uid].multi_level_switch_property[uid_info['UID']] = \
-            MultiLevelSwitchProperty(session=self._session,
-                                     gateway=self.gateway,
-                                     mydevolo=self._mydevolo,
-                                     element_uid=uid_info['UID'],
-                                     value=uid_info['properties']['value'],
-                                     switch_type=uid_info['properties']['switchType'],
-                                     max=uid_info['properties']['max'],
-                                     min=uid_info['properties']['min'])
+        self._logger.debug("Adding multi level switch property %s to %s.", uid_info['UID'], device_uid)
+        self.devices[device_uid].multi_level_switch_property[uid_info['UID']] = MultiLevelSwitchProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_multi_level_switch,
+            value=uid_info['properties']['value'],
+            switch_type=uid_info['properties']['switchType'],
+            max=uid_info['properties']['max'],
+            min=uid_info['properties']['min'])
 
     def _parameter(self, uid_info: dict):
         """ Process custom parameter setting (cps) properties."""
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding parameter settings to {device_uid}.")
-        self.devices[device_uid].settings_property['param_changed'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             param_changed=uid_info['properties']['paramChanged'])
+        self._logger.debug("Adding parameter settings to %s.", device_uid)
+        self.devices[device_uid].settings_property['param_changed'] = SettingsProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_setting,
+            param_changed=uid_info['properties']['paramChanged'])
 
     def _protection(self, uid_info: dict):
         """ Process protection setting (ps) properties. """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding protection settings to {device_uid}.")
-        self.devices[device_uid].settings_property['protection'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             local_switching=uid_info['properties']['localSwitch'],
-                             remote_switching=uid_info['properties']['remoteSwitch'])
+        self._logger.debug("Adding protection settings to %s.", device_uid)
+        self.devices[device_uid].settings_property['protection'] = SettingsProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_setting,
+            local_switching=uid_info['properties']['localSwitch'],
+            remote_switching=uid_info['properties']['remoteSwitch'])
 
     def _remote_control(self, uid_info: dict):
         """ Process remote control properties. """
         device_uid = get_device_uid_from_element_uid(uid_info['UID'])
-        self._logger.debug(f"Adding remote control to {device_uid}")
+        self._logger.debug("Adding remote control to %s.", device_uid)
         if not hasattr(self.devices[device_uid], "remote_control_property"):
             self.devices[device_uid].remote_control_property = {}
-        self.devices[device_uid].remote_control_property[uid_info['UID']] = \
-            RemoteControlProperty(session=self._session,
-                                  gateway=self.gateway,
-                                  mydevolo=self._mydevolo,
-                                  element_uid=uid_info['UID'],
-                                  key_count=uid_info['properties']['keyCount'],
-                                  key_pressed=uid_info['properties']['keyPressed'],
-                                  type=uid_info['properties']['type'])
+        self.devices[device_uid].remote_control_property[uid_info['UID']] = RemoteControlProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_remote_control,
+            key_count=uid_info['properties']['keyCount'],
+            key_pressed=uid_info['properties']['keyPressed'],
+            type=uid_info['properties']['type'])
 
     def _switch_type(self, uid_info: dict):
         """
@@ -481,28 +415,24 @@ class HomeControl(Mprm):
         switch with four buttons reports a switchType of 2. This confusing behavior is corrected by doubling the value.
         """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding switch type setting to {device_uid}")
-        self.devices[device_uid].settings_property['switch_type'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             value=uid_info['properties']['switchType'] * 2)
+        self._logger.debug("Adding switch type setting to %s.", device_uid)
+        self.devices[device_uid].settings_property['switch_type'] = SettingsProperty(element_uid=uid_info['UID'],
+                                                                                     setter=self.set_setting,
+                                                                                     value=uid_info['properties']['switchType']
+                                                                                     * 2)
 
     def _temperature_report(self, uid_info: dict):
         """ Process temperature report setting (trs) properties. """
         device_uid = get_device_uid_from_setting_uid(uid_info['UID'])
-        self._logger.debug(f"Adding temperature report settings to {device_uid}.")
-        self.devices[device_uid].settings_property['temperature_report'] = \
-            SettingsProperty(session=self._session,
-                             gateway=self.gateway,
-                             mydevolo=self._mydevolo,
-                             element_uid=uid_info['UID'],
-                             temp_report=uid_info['properties']['tempReport'],
-                             target_temp_report=uid_info['properties']['targetTempReport'])
+        self._logger.debug("Adding temperature report settings to %s.", device_uid)
+        self.devices[device_uid].settings_property['temperature_report'] = SettingsProperty(
+            element_uid=uid_info['UID'],
+            setter=self.set_setting,
+            temp_report=uid_info['properties']['tempReport'],
+            target_temp_report=uid_info['properties']['targetTempReport'])
 
     def _unknown(self, uid_info: dict):
         """ Ignore unknown properties. """
         ignore = ("devolo.SirenBinarySensor", "devolo.SirenMultiLevelSensor", "ss", "mcs")
         if not uid_info['UID'].startswith(ignore):
-            self._logger.debug(f"Found an unexpected element uid: {uid_info.get('UID')}")
+            self._logger.debug("Found an unexpected element uid: %s", uid_info['UID'])
