@@ -1,20 +1,19 @@
 import pytest
-
-from devolo_home_control_api.mydevolo import Mydevolo, GatewayOfflineError, WrongUrlError, WrongCredentialsError
+from devolo_home_control_api.mydevolo import GatewayOfflineError, Mydevolo, WrongCredentialsError, WrongUrlError
 
 
 class TestMydevolo:
-    @pytest.mark.usefixtures("mock_mydevolo__call")
+
     def test_credentials_valid(self, mydevolo):
         assert mydevolo.credentials_valid()
 
-    @pytest.mark.usefixtures("mock_mydevolo__call_raise_WrongCredentialsError")
+    @pytest.mark.usefixtures("mock_mydevolo_uuid_raise_WrongCredentialsError")
     def test_credentials_invalid(self, mydevolo):
         assert not mydevolo.credentials_valid()
 
     @pytest.mark.usefixtures("mock_mydevolo__call")
     def test_gateway_ids(self, mydevolo):
-        assert mydevolo.get_gateway_ids() == [self.gateway.get("id")]
+        assert mydevolo.get_gateway_ids() == [self.gateway['id']]
 
     @pytest.mark.usefixtures("mock_mydevolo__call")
     def test_gateway_ids_empty(self, mydevolo):
@@ -23,23 +22,23 @@ class TestMydevolo:
 
     @pytest.mark.usefixtures("mock_mydevolo__call")
     def test_get_full_url(self, mydevolo):
-        full_url = mydevolo.get_full_url(self.gateway.get("id"))
-        assert full_url == self.gateway.get("full_url")
+        full_url = mydevolo.get_full_url(self.gateway['id'])
+        assert full_url == self.gateway['full_url']
 
     @pytest.mark.usefixtures("mock_mydevolo__call")
     def test_get_gateway(self, mydevolo):
-        details = mydevolo.get_gateway(self.gateway.get("id"))
-        assert details.get("gatewayId") == self.gateway.get("id")
+        details = mydevolo.get_gateway(self.gateway['id'])
+        assert details.get("gatewayId") == self.gateway['id']
 
     @pytest.mark.usefixtures("mock_mydevolo__call_raise_WrongUrlError")
     def test_get_gateway_invalid(self, mydevolo):
         with pytest.raises(WrongUrlError):
-            mydevolo.get_gateway(self.gateway.get("id"))
+            mydevolo.get_gateway(self.gateway['id'])
 
     @pytest.mark.usefixtures("mock_mydevolo__call")
     def test_get_zwave_products(self, mydevolo):
-        product = mydevolo.get_zwave_products(manufacturer="0x0060", product_type="0x0001", product="0x000")
-        assert product.get("name") == "Everspring PIR Sensor SP814"
+        device_info = mydevolo.get_zwave_products(manufacturer="0x0060", product_type="0x0001", product="0x000")
+        assert device_info.get("name") == "Everspring PIR Sensor SP814"
 
     @pytest.mark.usefixtures("mock_mydevolo__call_raise_WrongUrlError")
     def test_get_zwave_products_invalid(self, mydevolo):
@@ -47,67 +46,48 @@ class TestMydevolo:
         assert device_info.get("name") == "Unknown"
 
     @pytest.mark.usefixtures("mock_mydevolo__call")
-    def test_maintenance_on(self, mydevolo):
-        assert not mydevolo.maintenance()
-
-    @pytest.mark.usefixtures("mock_mydevolo__call")
-    def test_maintenance_off(self, mydevolo):
-        assert mydevolo.maintenance()
+    @pytest.mark.parametrize("result", [True, False])
+    def test_maintenance(self, mydevolo, result):
+        assert mydevolo.maintenance() == result
 
     def test_set_password(self, mydevolo):
-        mydevolo._gateway_ids = [self.gateway.get("id")]
-
-        mydevolo.password = self.user.get("password")
+        mydevolo._gateway_ids = [self.gateway['id']]
+        mydevolo.password = self.user['password']
 
         assert mydevolo._uuid is None
         assert mydevolo._gateway_ids == []
 
     def test_set_user(self, mydevolo):
-        mydevolo._gateway_ids = [self.gateway.get("id")]
-
-        mydevolo.user = self.user.get("username")
+        mydevolo._gateway_ids = [self.gateway['id']]
+        mydevolo.user = self.user['username']
 
         assert mydevolo._uuid is None
         assert mydevolo._gateway_ids == []
 
     def test_get_user(self, mydevolo):
-        mydevolo.user = self.user.get("username")
-        assert mydevolo.user == self.user.get("username")
+        mydevolo.user = self.user['username']
+        assert mydevolo.user == self.user['username']
 
     def test_get_password(self, mydevolo):
-        mydevolo.password = self.user.get("password")
-        assert mydevolo.password == self.user.get("password")
-
-    def test_singleton_mydevolo(self):
-        with pytest.raises(SyntaxError):
-            Mydevolo.get_instance()
-
-        first = Mydevolo()
-        with pytest.raises(SyntaxError):
-            Mydevolo()
-
-        second = Mydevolo.get_instance()
-        assert first is second
-        Mydevolo.del_instance()
+        mydevolo.password = self.user['password']
+        assert mydevolo.password == self.user['password']
 
     @pytest.mark.usefixtures("mock_mydevolo__call")
     def test_uuid(self, mydevolo):
         mydevolo._uuid = None
-        assert mydevolo.uuid() == self.user.get("uuid")
+        assert mydevolo.uuid() == self.user['uuid']
 
     @pytest.mark.usefixtures("mock_response_wrong_credentials_error")
     def test_call_WrongCredentialsError(self):
         mydevolo = Mydevolo()
         with pytest.raises(WrongCredentialsError):
             mydevolo._call("test")
-        Mydevolo.del_instance()
 
     @pytest.mark.usefixtures("mock_response_wrong_url_error")
     def test_call_WrongUrlError(self):
         mydevolo = Mydevolo()
         with pytest.raises(WrongUrlError):
             mydevolo._call("test")
-        Mydevolo.del_instance()
 
     @pytest.mark.usefixtures("mock_response_gateway_offline")
     def test_call_GatewayOfflineError(self, mydevolo):
